@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.5.2.8 2007/10/12 04:32:14 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.25 2007/11/14 03:59:58 y-mori Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -57,12 +57,6 @@
 /* pid file name */
 #define PID_FILE_NAME "pgpool.pid"
 
-/* strict mode comment in SQL */
-#define STRICT_MODE_STR "/*STRICT*/"
-#define STRICT_MODE(s) (strncasecmp((s), STRICT_MODE_STR, strlen(STRICT_MODE_STR)) == 0)
-#define NO_STRICT_MODE_STR "/*NO STRICT*/"
-#define NO_STRICT_MODE(s) (strncasecmp((s), NO_STRICT_MODE_STR, strlen(NO_STRICT_MODE_STR)) == 0)
-
 typedef enum {
 	POOL_CONTINUE = 0,
 	POOL_IDLE,
@@ -72,6 +66,13 @@ typedef enum {
 	POOL_DEADLOCK
 } POOL_STATUS;
 
+
+typedef enum {
+	INIT_CONFIG = 1,   /* 0x01 */
+	RELOAD_CONFIG = 2  /* 0x02 */
+} POOL_CONFIG_CONTEXT;
+
+
 /* protocol major version numbers */
 #define PROTO_MAJOR_V2	2
 #define PROTO_MAJOR_V3	3
@@ -127,6 +128,8 @@ typedef struct {
     int	child_life_time;	/* if idle for this seconds, child exits */
     int	connection_life_time;	/* if idle for this seconds, connection closes */
     int	child_max_connections;	/* if max_connections received, child exits */
+	int client_idle_limit;		/* If client_idle_limit is n (n > 0), the client is forced to be
+								   disconnected after n seconds idle */
 	int authentication_timeout; /* maximum time in seconds to complete client authentication */
     int	max_pool;	/* max # of connection pool per child */
     char *logdir;		/* logging directory */
@@ -160,6 +163,12 @@ typedef struct {
 	int health_check_timeout;	/* health check timeout */
 	int health_check_period;	/* health check period */
 	char *health_check_user;		/* PostgreSQL user name for health check */
+	char *failover_command;     /* execute command when failover happens */
+	char *failback_command;     /* execute command when failback happens */
+	char *recovery_user;		/* PostgreSQL user name for online recovery */
+	char *recovery_password;		/* PostgreSQL user password for online recovery */
+	char *recovery_1st_stage_command;   /* Online recovery command in 1st stage */
+	char *recovery_2nd_stage_command;   /* Online recovery command in 2nd stage */
 	int insert_lock;	/* if non 0, automatically lock table with INSERT to keep SERIAL
 						   data consistency */
 	int ignore_leading_white_space;		/* ignore leading white spaces of each query */
@@ -306,7 +315,7 @@ typedef struct {
 					   pool_config->backend_desc->num_backends))
 #endif
 #define NUM_BACKENDS (in_load_balance? (selected_slot+1) : \
-					  (((!REPLICATION && !PARALLEL_MODE) && !MASTER_SLAVE) ? Req_info->master_node_id+1: \
+					  (((!REPLICATION && !PARALLEL_MODE) && !MASTER_SLAVE)? Req_info->master_node_id+1: \
 					   pool_config->backend_desc->num_backends))
 #define BACKEND_INFO(backend_id) (pool_config->backend_desc->backend_info[(backend_id)])
 #define LOAD_BALANCE_STATUS(backend_id) (pool_config->load_balance_status[(backend_id)])
@@ -341,6 +350,8 @@ typedef struct {
 #define NO_LOCK_COMMENT_SZ (sizeof(NO_LOCK_COMMENT)-1)
 
 #define MAX_NUM_SEMAPHORES		3
+#define CONN_COUNTER_SEM 0
+#define REQUEST_INFO_SEM 1
 
 #define MY_PROCESS_INFO (pids[my_proc_id])
 
@@ -359,18 +370,22 @@ typedef enum SemNum
  */
 typedef enum {
 	NODE_UP_REQUEST = 0,
-	NODE_DOWN_REQUEST
+	NODE_DOWN_REQUEST,
+	NODE_RECOVERY_REQUEST,
+	CLOSE_IDLE_REQUEST
 } POOL_REQUEST_KIND;
 
 typedef struct {
 	POOL_REQUEST_KIND	kind;	/* request kind */
-	int node_id;		/* request node id */
+	int node_id[MAX_NUM_BACKENDS];		/* request node id */
 	int master_node_id;	/* the youngest node id which is not in down status */
+	int conn_counter;
 } POOL_REQUEST_INFO;
 
 /*
  * global variables
  */
+extern pid_t mypid; /* parent pid */
 extern POOL_CONFIG *pool_config;	/* configuration values */
 extern POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
 extern volatile sig_atomic_t backend_timer_expired; /* flag for connection closed timer is expired */
@@ -382,15 +397,20 @@ extern int in_load_balance;		/* non 0 if in load balance mode */
 extern int selected_slot;		/* selected DB node for load balance */
 extern int master_slave_dml;	/* non 0 if master/slave mode is specified in config file */
 extern POOL_REQUEST_INFO *Req_info;
+extern volatile sig_atomic_t *InRecovery;
 extern char remote_ps_data[];		/* used for set_ps_display */
+extern volatile sig_atomic_t got_sighup;
 
 /*
  * public functions
  */
+extern char *get_config_file_name(void);
+extern char *get_hba_file_name(void);
 extern void pool_error(const char *fmt,...);
 extern void pool_debug(const char *fmt,...);
 extern void pool_log(const char *fmt,...);
-extern int pool_get_config(char *confpath);
+extern int pool_init_config(void);
+extern int pool_get_config(char *confpath, POOL_CONFIG_CONTEXT context);
 extern void do_child(int unix_fd, int inet_fd);
 extern void pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file);
 extern int select_load_balancing_node(void);
@@ -403,6 +423,7 @@ extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 extern POOL_CONNECTION *pool_open(int fd);
 extern void pool_close(POOL_CONNECTION *cp);
 extern int pool_read(POOL_CONNECTION *cp, void *buf, int len);
+extern int pool_read_parallel(POOL_CONNECTION *cp, void *buf, int len);
 extern char *pool_read2(POOL_CONNECTION *cp, int len);
 extern int pool_write(POOL_CONNECTION *cp, void *buf, int len);
 extern int pool_flush(POOL_CONNECTION *cp);
@@ -427,6 +448,7 @@ extern POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend,
 								  POOL_CONNECTION_POOL *backend);
 
 extern void notice_backend_error(int node_id);
+extern void degenerate_backend_set(int *node_id_set, int count);
 extern void send_failback_request(int node_id);
 
 extern void pool_connection_pool_timer(POOL_CONNECTION_POOL *backend);
@@ -471,6 +493,9 @@ extern int send_startup_packet(POOL_CONNECTION_POOL_SLOT *cp);
 extern void pool_free_startup_packet(StartupPacket *sp);
 extern void child_exit(int code);
 
+extern void connection_count_up(void);
+extern void connection_count_down(void);
+
 extern int health_check(void);
 extern int system_db_health_check(void);
 
@@ -496,6 +521,7 @@ extern POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 /* define pool_system.c */
 extern POOL_CONNECTION_POOL_SLOT *pool_system_db_connection(void);
 extern DistDefInfo *pool_get_dist_def_info (char * dbname, char * schema_name, char * table_name);
+extern RepliDefInfo *pool_get_repli_def_info (char * dbname, char * schema_name, char * table_name);
 extern int pool_get_id (DistDefInfo *info, const char * value);
 extern int system_db_connect (void);
 extern int pool_memset_system_db_info (SystemDBInfo *info);
@@ -525,4 +551,12 @@ extern void init_ps_display(const char *username, const char *dbname,
 extern void set_ps_display(const char *activity, bool force);
 extern const char *get_ps_display(int *displen);
 
+/* recovery.c */
+extern int start_recovery(int recovery_node);
+extern void finish_recovery(void);
+
+/* child.c */
+extern void pool_set_nonblock(int fd);
+extern void pool_unset_nonblock(int fd);
+
 #endif /* POOL_H */
