@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.6.2.3 2007/07/26 10:12:01 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.9 2007/07/26 10:15:45 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -27,6 +27,9 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <sys/un.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 #ifdef HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>
 #endif
@@ -140,7 +143,7 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor, in
 						free(CONNECTION_SLOT(p, j));
 					}
 					info = p->info;
-					memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+					memset(p, 0, sizeof(POOL_CONNECTION_POOL_SLOT));
 					p->info = info;
 					memset(p->info, 0, sizeof(ConnectionInfo));
 					return NULL;
