@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_sema.c,v 1.1.1.1 2006/09/08 03:36:09 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_sema.c,v 1.3 2007/09/26 03:32:14 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Portions Copyright (c) 2003-2006, PgPool Global Development Group
+ * Portions Copyright (c) 2003-2007, PgPool Global Development Group
  * Portions Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
@@ -75,7 +75,7 @@ IpcSemaphoreKill(int status, Datum semId)
 int
 pool_semaphore_create(int numSems)
 {
-	int			semNum;
+	int			i;
 
 	/* Try to create new semaphore set */
 	semId = semget(IPC_PRIVATE, numSems, IPC_CREAT | IPC_EXCL | IPCProtection);
@@ -89,15 +89,15 @@ pool_semaphore_create(int numSems)
 	on_shmem_exit(IpcSemaphoreKill, semId);
 
 	/* Initialize it to count 1 */
-	for (semNum = 0; semNum < MAX_NUM_SEMAPHORES; semNum++)
+	for (i = 0; i < numSems; i++)
 	{
 		union semun semun;
 
 		semun.val = 1;
-		if (semctl(semId, 0, SETVAL, semun) < 0)
+		if (semctl(semId, i, SETVAL, semun) < 0)
 		{
 			pool_error("semctl(%d, %d, SETVAL, %d) failed: %s",
-					   semId, semNum, 1);
+					   semId, i, 1);
 			return -1;
 		}
 	}
